#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rand.h"
#include "types.h"

#define pi 3.141592653589793238462643
#define	epsilon 1e-10
#define	infinity 6e66

/* window creation */
void cubewalls(pattern*);
void spherewalls(pattern*);
void clusterwalls(pattern*,parlist*);

/* creation and removal of bodies */
void addballs(pattern*,int,double**,double);
void addball(body*, double*,double);
void addhole(body*, double*,double);
void addwall(body*, double*,double*);
void makeball(body*, double,double,double, double);
void makehole(body*, double,double,double, double);
void makewall(body*, double,double,double, double,double,double);
void removebody(body*);

/* -------------------------- */
/*                            */
/* create some windows        */
/* -------------------------- */
void cubewalls(pattern*win)
{
  win->n = 6;
  win->nwin = 6;

  win->win = (body*)calloc(win->n,sizeof(body));
  win->b = (body*)NULL;
  makewall(&win->win[0],  1,.5,.5, +1, 0, 0); win->win[0].n = 0;
  makewall(&win->win[1],  0,.5,.5, -1, 0, 0); win->win[1].n = 1;
  makewall(&win->win[2], .5, 1,.5,  0,+1, 0); win->win[2].n = 2;
  makewall(&win->win[3], .5, 0,.5,  0,-1, 0); win->win[3].n = 3;
  makewall(&win->win[4], .5,.5, 1,  0, 0,+1); win->win[4].n = 4;
  makewall(&win->win[5], .5,.5, 0,  0, 0,-1); win->win[5].n = 5;
}
void spherewalls(pattern*win)
{
  win->n = 1;
  win->nwin = 1;

  win->win = (body*)calloc(win->n,sizeof(body));
  win->b = (body*)NULL;
  makehole(&win->win[0], .5,.5,.5, .5); win->win[0].n = 0;
  /* makewall(&win->win[1], .5,.5,.5, 0,0,-1); win->win[1].n = 1; */
}
void clusterwalls(pattern*win, parlist*par)
{
  int i, nzoa=21;    /* the number of planes for the zoa  */
                     /* this is sufficient for 30 degrees */
                     /* check it if you change the cone   */
  double theta, phi, tiny=1e-6;	/* to avoid the nasty tip of the cone */

  /* the sphere */
  win->win = (body*)calloc(1,sizeof(body));
  win->n = 1; win->nwin = 1;
  win->b = (body*)calloc(win->n,sizeof(body));
  makehole(&win->win[0], .5,.5,.5, .5); win->win[0].n = 0;

  /* where to put the zone of avoidance */
  /* we cut out 30 degrees on each side of the zone of avoidance */
  switch(par->hemisphere) {
  case 1 : theta = 150.*pi/180.; break; /* northern */
  case 2 : theta =  30.*pi/180.; break; /* southern */
  default : return; break;	/* no zoa; complete sphere */
  }

  /* walls for the zone of avoidance */
  win->n += nzoa; win->nwin += nzoa;
  win->win = (body*)realloc(win->win,win->n*sizeof(body));
  for(i=1, phi=0; i<=nzoa; i++, phi+=2*pi/nzoa) {
    makewall(&win->win[i], .5+tiny*rand1(),.5+tiny*rand1(),.5+tiny*rand1(), 
	     sin(theta)*cos(phi), sin(theta)*sin(phi), cos(theta)); 
    win->win[i].n = i;
  }  

  return;
}

/* ------------------------------------ */
/*                                      */
/* fill structure of geometric objects  */
/* with appropriate parameters          */
/* ------------------------------------ */
void addballs(pattern*old, int nnew, double**new, double r)
{
  int j;

  old->n = old->nwin+nnew; 
  old->b = (body*)realloc(old->b,old->n*sizeof(body));
  for(j = old->nwin; j < old->n; j++) {
    addball(&old->b[j],new[j- old->nwin],r);
    old->b[j].n = j;
  }
}
void addball(body*new, double*center, double radius)
{
  int i;

  new->type = ball;
  new->v = (double*)calloc(4,sizeof(double));
  new->touch = (body*)NULL;

  new->par = (double**)calloc(3,sizeof(double*));
  new->par[0] = (double*)calloc(3,sizeof(double));
  for(i=0; i<3; i++)new->par[0][i] = center[i];
  new->par[1] = (double*)calloc(1,sizeof(double));
  new->par[1][0] = radius;
}
void addhole(body*new, double*center, double radius)
{
  int i;

  new->type = hole;
  new->v = (double*)calloc(4,sizeof(double));
  new->touch = (body*)NULL;

  new->par = (double**)calloc(3,sizeof(double*));
  new->par[0] = (double*)calloc(3,sizeof(double));
  for(i=0; i<3; i++)new->par[0][i] = center[i];
  new->par[1] = (double*)calloc(1,sizeof(double));
  new->par[1][0] = radius;
}
void addwall(body*new, double*point, double*normal)
{
  int i;

  new->type = wall;
  new->v = (double*)calloc(4,sizeof(double));
  new->touch = (body*)NULL;

  new->par = (double**)calloc(3,sizeof(double*));
  new->par[0] = (double*)calloc(3,sizeof(double));
  for(i=0; i<3; i++)new->par[0][i] = point[i];
  new->par[1] = (double*)calloc(3,sizeof(double));
  for(i=0; i<3; i++)new->par[1][i] = normal[i];
}

void makeball(body*new, double c0,double c1,double c2, double radius)
{
  double c[3];
  c[0] = c0, c[1] = c1, c[2] = c2;
  addball(new,c,radius);
}
void makehole(body*new, double c0,double c1,double c2, double radius)
{
  double c[3];
  c[0] = c0, c[1] = c1, c[2] = c2;
  addhole(new,c,radius);
}
void makewall(body*new, double p0,double p1,double p2,
	      double n0,double n1,double n2)
{
  double p[3],n[3];
  p[0] = p0, p[1] = p1, p[2] = p2;
  n[0] = n0, n[1] = n1, n[2] = n2;
  addwall(new,p,n);
}

void removebody(body*b)
{
  free((void*)b->v);
  free((void*)b->par[0]); free((void*)b->par[1]); free((void*)b->par);
  b->type = nobody;
}
