#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "types.h"
#include "rand.h"

#define readint atoi(ar[n][2] ? &ar[n][2] : ar[++n])
#define readflt atof(ar[n][2] ? &ar[n][2] : ar[++n])
#define readstr (ar[n][2] ? &ar[n][2] : ar[++n])

/* Declarations */

void init_params(parlist*, char**, int);
void Explain(FILE*, char*, parlist*);

/* ----------------------------------------------------------- */
/*                                                             */
/* initialize the parameters, evaluate the commandline options */
/* ----------------------------------------------------------- */
void init_params(parlist*par, char *ar[], int nar)
{
  int n=0;

  /* assign default values */
  par->lorad = 0, par->hirad = .25, par->drad = .025;
  par->n = 10, par->length = 1;
  par->rmbound = true;
  par->outfile = "quark.res"; par->infile = "nofile";

  par->hemisphere = 0;
  par->vmax = 10000;
  par->subsample = 1;
  par->scale = 1;
  par->nin = par->n;

  /* work through arguments */
  while(++n < nar)
    if((ar[n][0] == '-') && (ar[n][1]))
      switch(ar[n][1])
	{
	case'r' : par->lorad      = readflt; break;
	case'R' : par->hirad      = readflt; break;
	case'd' : par->drad       = readflt; break;
	case'n' : par->n          = readint; break;
	case'B' : par->rmbound    = false;    break;
	case'l' : par->length     = readint; break;
	case'o' : par->outfile    = readstr; break;
	case'i' : par->infile     = readstr; break;
	case'N' : par->nin        = readint; break;

	case'H' : par->hemisphere = readint; break;
	case'v' : par->vmax       = readflt; break;
	case's' : par->subsample  = readflt; break;
	case'S' : par->scale      = readflt; break;
	default : Explain(stderr,ar[0],par);
	}
    else
      Explain(stderr,ar[0],par);

  /* open output file */
  par->outfp = fopen(par->outfile,"w");

  return;
}


/* --------------- */
/* explain options */
/* --------------- */
void Explain(FILE*fp, char*name, parlist*act)
{
  fprintf(fp,"# %s options\n"
	  "# \t-r   lowest radius of balls            (actual %g)\n"
	  "# \t-R   highest radius of balls           (actual %g)\n"
	  "# \t-d   step of radius of balls           (actual %g)\n"
	  "# \t-n   number of points actually used    (actual %d)\n"
	  "# \t-l   sample length                     (actual %d)\n"
	  "# \t-B   perform boundary deconvolution    (actual %s)\n"
	  "# \t-o   output file name                  (actual %s)\n"
	  "# \t-i   input file name                   (actual %s)\n"
	  "# \t-N   number of points in input file    (actual %d)\n"
	  "#\n"
	  "# \t-H   hemipshere                        (actual %s)\n"
	  "# \t-s   percentage for subsampling        (actual %g)\n"
	  "# \t-S   used scaling factor s,            (actual %g)\n"
	  "# \t     resulting in (v0, v1/s, v2/s^2, v3/s^3)\n" ,
	  name, act->lorad, act->hirad, act->drad, 
	  act->n, act->length, 
	  (act->rmbound ? "true" : "false"), 
	  act->outfile, act->infile, act->nin,
	  (act->hemisphere==1 ? "northern" 
	   : (act->hemisphere==2 ? "southern" : "none")),
	  act->subsample, act->scale
	  );

  if(fp == stderr)exit(0);
  else return;
}
